/*
 * File: bytecode.c
 * Author: Saumay Debray
 * Purpose: Defines the byte code for a program -- in this case,
 *	binary search.
 */

#include "interp.h"

/*
 * in this program, registers are used as follows:
 *
 *	r0 : lo
 *	r1 : hi
 *	r2 : mid
 *	r3 : key  (the value being searched for)
 *	r4 : scratch
 */
Instr byteCodeArray[] = {
  {MOV,
   {MEM, 2},
   {NONE, 0},
   {REG, 0}
  },			// Instr 0: lo := Memory[2] 
  {MOV,
   {MEM, 3},
   {NONE, 0},
   {REG, 1}
  },			// Instr 1: hi := Memory[3]
  {MOV,
   {MEM, 0},
   {NONE, 0},
   {REG, 3}
  },			// Instr 2: key := Memory[0]
  {IF_GT,
   {REG, 0},
   {REG, 1},
   {IMM, 13}
  },			// Instr 3: if lo > hi goto Instr13
  {ADD,
   {REG, 0},
   {REG, 1},
   {REG, 4}
  },			// Instr 4: tmp := lo + hi
  {DIV,
   {REG, 4},
   {IMM, 2},
   {REG, 2}
  },			// Instr 5: mid := tmp/2
  {MOV,
   {INDIR, 2},
   {NONE, 0},
   {REG, 4}
  },			// Instr 6: tmp := A[mid]
  {IF_EQ,
   {REG, 3},
   {REG, 4},
   {IMM, 14}
  },			// Instr 7: if key == tmp goto Instr14
  {IF_LT,
   {REG, 3},
   {REG, 4},
   {IMM, 11}
  },			// Instr 8: if key < tmp goto Instr11
  {ADD,
   {REG, 2},
   {IMM, 1},
   {REG, 0}
  },			// Instr  9: lo := mid + 1
  {JMP,
   {IMM, 3},
   {NONE, 0},
   {NONE, 0}
  },			// Instr 10: goto Instr3
  {SUB,
   {REG, 2},
   {IMM, 1},
   {REG, 1}
  },			// Instr 11: hi := mid - 1
  {JMP,
   {IMM, 3},
   {NONE, 0},
   {NONE, 0}
  },			// Instr 12: goto Instr3
  {MOV,
   {IMM, 3},
   {NONE, 0},
   {REG, 2}
  },			// Instr 13: mid := 3
  {SUB,
   {REG, 2},
   {IMM, 4},
   {REG, 2}
  },			// Subtract out the start offset of the array
  {PRINT,
   {REG, 2},
   {NONE, 0},
   {NONE, 0}
  },			// Instr 14: print mid
  {HALT,
   {NONE, 0},
   {NONE, 0},
   {NONE, 0}
  }			// halt
};

int nInstrs = sizeof(byteCodeArray)/sizeof(Instr);
