#ifndef _TRACE_READER_H_
#define _TRACE_READER_H_

#define _FILE_OFFSET_BITS 64

#ifdef TARGET_WINDOWS

#define ftell _ftelli64

#define fseek _fseeki64

#else

#define ftell ftello

#define fseek fseeko

#endif

#include <stdint.h>
#include <xed-interface.h>
#include "../shared/LynxReg.h"
#include "../shared/header.h"

typedef struct {
    uint8_t addrGen;
    uint8_t seg;
    uint8_t base;
    uint8_t index;
    uint32_t scale;
    uint64_t disp;
    uint64_t addr;
    uint8_t size;
} ReaderMemOp;

typedef enum {
    NONE_OP,
    REG_OP,
    MEM_OP,
    UNSIGNED_IMM_OP,
    SIGNED_IMM_OP
} ReaderOpType;

typedef struct ReaderOp_t {
    uint8_t mark;
    ReaderOpType type;
    union {
        LynxReg reg;
        ReaderMemOp mem;
        uint64_t unsignedImm;
        int64_t signedImm;
    };
    struct ReaderOp_t *next;
} ReaderOp;

typedef struct {
    ReaderOp srcOps[3];
    uint8_t srcOpCnt;
    ReaderOp dstOps[1];
    uint8_t dstOpCnt;
    ReaderOp readWriteOps[1];
    uint8_t readWriteOpCnt;
    char mnemonic[128];
    xed_iclass_enum_t insClass;
} InsInfo;

//maybe include the bytelength
typedef struct {
    uint8_t binary[15];
    uint8_t binSize;
    uint64_t addr;
    uint32_t fnId;
    uint32_t srcId;
    uint32_t tid;
} ReaderIns;

typedef struct {
    ExceptionType type;		// The type of exception
    uint32_t code;		// The Linux or Windows code for the exception
    uint64_t addr;		// The address the exception executed at
    uint32_t tid;		// The thread id of the exception
} ReaderException;

typedef enum {
    EXCEPTION_EVENT,
    INS_EVENT
} ReaderEventType;

typedef struct {
    ReaderEventType type;		// The type of the event
    union
    {
	    ReaderIns ins;	// A union containing a data struct based on the type of event
	    ReaderException exception;
    };
//    uint64_t eid;		// The numeric id of the event in the trace, that is the number of the event (the first event in the trace being 0)
} ReaderEvent;

void initReader (char *filename, uint32_t debug);

void closeReader ();

uint32_t nextEvent(ReaderEvent *event);

uint32_t nextEventWithCheck(ReaderEvent *nextEvent, InsInfo *curInfo);

void getMemoryVal (uint64_t addr, uint32_t size, uint8_t *buf);

const uint8_t *getRegisterVal (LynxReg reg, uint32_t thread);

const char *fetchSrcName(ReaderIns *ins);

const char *fetchFnName(ReaderIns *ins);

void initInsInfo(InsInfo *info);

int fetchInsInfo(ReaderIns *ins, InsInfo *info);

uint64_t getCurEid();

#endif // _TRD_H_
