# Reader #

Welcome! In this folder you will find the code necessary to build the toolset's trace reader. The reader will be used to read a trace in from the binary format used by the tracer so that programs can analyze the trace. It's design allows the user to walk through a trace instruction by instruction and make queries to the shadow state maintained by the reader.

## Prerequesites ##

Linux

* [Intel's XED disassembler](https://software.intel.com/en-us/articles/xed-x86-encoder-decoder-software-library)

## Building ##

1. Download XED from [XED's webpage](https://software.intel.com/en-us/articles/xed-x86-encoder-decoder-software-library)
2. Extract XED from the archive. The location of XED's base folder will henceforth be referred to as $XED_BASE
3. Edit the Makefile in this directory and change the value of XEDPATH to $XED_BASE
4. Run make

## API ##
To use the library, you'll have to include Reader.h. This file contains the API and all of the necessary structs you'll need. Below there is some more information about the API.

void initReader(filename, debug)
* This function must be called before ANY other function in the reader. It will set up the internal state of the reader so that the file located at filename may be read.

void closeReader()
* This function is intended to be called after reading the trace is complete. It closes the trace file and frees the shadow architecture. The initInsInfo may still be called after this point to get information about an instruction

uint32_t nextEvent(ReaderEvent *event)
* The nextEvent function will fetch the next event from a trace and store it in the ReaderEvent struct passed in as an arguement. It will return a 1 if an instruction was read from the trace and 0 if the end of the trace has been reached

uint32_t nextEventWithCheck(ReaderEvent *nextEvent, InsInfo *curInfo)
* The previous function and this function perform very similar operations. In addition to everything that nextEvent does, it also goes through checks to ensure that the shadow architecture is consistent. To do this, it requires the InsInfo of the current instruction to check against data read from the trace, so make sure you pass in a container for the next instruction, and the InsInfo from the current instruction.

void getMemoryVal(uint64_t addr, uint32_t size, uint8_t *buf)
* To get a region of memory from the shadow memory, use this function. One thing to watch out for is that the function expects that buf will have at least size bytes available to store the region into.

const uint8_t *getRegisterVal(LynxReg reg, uint32_t thread)
* Similar to getMemoryVal, this API allows you to get a register from the shadow registers.

const char *fetchSrcName(ReaderIns *ins)
* Fetches the source name that corresponds to the source ID stored in the ReaderIns.

const char *fetchFnName(ReaderIns *ins)
* Fetches the function name that corresponds to the function ID stored in the ReaderIns.

void initInsInfo(InsInfo *info)
* Initializes the initInsInfo struct. This needs to be done before InsInfo can be used because, despite the way it looks, the Ops are not stored in an array. They are stored in a partially statically defined linked list so that we rarely need to dynamically allocate more memory. Thus one of the big responsibilities of initInsInfo is to set up this list. One important side effect of this is you can't loop until next is null like most linked lists, so when looping over all of the ops make sure you use the corresponding count field.

int fetchInsInfo(ReaderIns *ins, InsInfo *info)
* Retrieves additional information about the instruction from the disassembler. This information includes the instruction's operands, textual representation and instruction type (i.e. push, pop, add, etc). Ensure that initInsInfo was called on the InsInfo struct passed into the second arguement before calling this function. Additionally 1 will be returned if an error was encountered, and 0 will otherwise be returned.

## Linking to the library ##

Since the reader a library, you'll have to write your own executable and compile against the library. To do so, you'll need to add -ltrd and -lxed to the link command. If the linker complains that it can't find those libraries, also add -L$THIS_DIR and -L$XED_BASE/lib to your link command.

## Examples ##

Currently the only example we have of using the reader is trace2ascii which is stored in the ScienceUpToPar/Tools/trace2ascii folder. It takes a trace in as an input, and prints out the ascii representation of the trace.

## Common Errors ##

 When trying to run an executable that uses the reader, you get the error './executable: error while loading shared libraries: libtrd.so: cannot open shared object file: No such file or directory'. 
* Add the path to the missing library to the environment variable $LD_LIBRARY_PATH (the same error may occur with libxed as well)

