#ifndef _LYNX_REG_H_
#define _LYNX_REG_H_
#include <stdint.h>

/* This enum represents all of the registers in x86 for all of the different
   architectures. Note that different architectures use different subsets of
   these registers and as a result, each of the register groups are named.
   There should never be an architecture that uses a subset of one of the
   groups in this file. Also, it should be noted that LynxReg is ordered so
   that full registers appear first, making it possibly to easily loop over
   the full registers*/
typedef enum {
	LYNX_FIRST=0,
	LYNX_ZMM_FIRST=LYNX_FIRST,
	LYNX_ZMM0=LYNX_ZMM_FIRST,
	LYNX_ZMM1,
	LYNX_ZMM2,
	LYNX_ZMM3,
	LYNX_ZMM4,
	LYNX_ZMM5,
	LYNX_ZMM6,
	LYNX_ZMM7,
	LYNX_ZMM8,
	LYNX_ZMM9,
	LYNX_ZMM10,
	LYNX_ZMM11,
	LYNX_ZMM12,
	LYNX_ZMM13,
	LYNX_ZMM14,
	LYNX_ZMM15,
	LYNX_ZMM16,
	LYNX_ZMM17,
	LYNX_ZMM18,
	LYNX_ZMM19,
	LYNX_ZMM20,
	LYNX_ZMM21,
	LYNX_ZMM22,
	LYNX_ZMM23,
	LYNX_ZMM24,
	LYNX_ZMM25,
	LYNX_ZMM26,
	LYNX_ZMM27,
	LYNX_ZMM28,
	LYNX_ZMM29,
	LYNX_ZMM30,
	LYNX_ZMM31,
	LYNX_ZMM_LAST=LYNX_ZMM31,
	LYNX_K_MASK_FIRST,
	LYNX_K0=LYNX_K_MASK_FIRST,
	LYNX_K1,
	LYNX_K2,
	LYNX_K3,
	LYNX_K4,
	LYNX_K5,
	LYNX_K6,
	LYNX_K7,
	LYNX_K_MASK_LAST=LYNX_K7,
	LYNX_GR64_FIRST,
	LYNX_RIP=LYNX_GR64_FIRST,
	LYNX_RAX,
	LYNX_RBX,
	LYNX_RCX,
	LYNX_RDX,
	LYNX_RBP,
	LYNX_RSI,
	LYNX_RDI,
	LYNX_RSP,
	LYNX_R8,
	LYNX_R9,
	LYNX_R10,
	LYNX_R11,
	LYNX_R12,
	LYNX_R13,
	LYNX_R14,
	LYNX_R15,
	LYNX_GR64_LAST=LYNX_R15,
	LYNX_SEG_FIRST,
	LYNX_SEG_CS=LYNX_SEG_FIRST,
	LYNX_SEG_DS,
	LYNX_SEG_ES,
	LYNX_SEG_FS,
	LYNX_SEG_GS,
	LYNX_SEG_SS,
	LYNX_SEG_LAST=LYNX_SEG_SS,
	LYNX_SSE_FLG_FIRST,
	LYNX_MXCSR=LYNX_SSE_FLG_FIRST,
	LYNX_SSE_FLG_LAST=LYNX_MXCSR,
	LYNX_FPU_FIRST,
	LYNX_ST0=LYNX_FPU_FIRST,
	LYNX_ST1,
	LYNX_ST2,
	LYNX_ST3,
	LYNX_ST4,
	LYNX_ST5,
	LYNX_ST6,
	LYNX_ST7,
	LYNX_FPU_LAST=LYNX_ST7,
	LYNX_FPU_STAT_FIRST,
	LYNX_FPSW=LYNX_FPU_STAT_FIRST,
	LYNX_FPCW,
	LYNX_FPTAG_FULL,
	LYNX_FPIP_OFF,
	LYNX_FPIP_SEL,
	LYNX_FPDP_OFF,
	LYNX_FPDP_SEL,
	LYNX_FPOPCODE,
	LYNX_FPU_STAT_LAST=LYNX_FPOPCODE,
	LYNX_DBG_FIRST,
	LYNX_DR0=LYNX_DBG_FIRST,
	LYNX_DR1,
	LYNX_DR2,
	LYNX_DR3,
	LYNX_DR4,
	LYNX_DR5,
	LYNX_DR6,
	LYNX_DR7,
	LYNX_DBG_LAST=LYNX_DR7,
	LYNX_CTRL_FIRST,
	LYNX_CR0=LYNX_CTRL_FIRST,
	LYNX_CR1,
	LYNX_CR2,
	LYNX_CR3,
	LYNX_CR4,
	LYNX_CTRL_LAST=LYNX_CR4,
	LYNX_FLAGS_FIRST,
	LYNX_GFLAGS=LYNX_FLAGS_FIRST,
	LYNX_FLAGS_LAST=LYNX_GFLAGS,
	LYNX_LAST_FULL=LYNX_GFLAGS,
	LYNX_YMM_X86_FIRST,
	LYNX_YMM0=LYNX_YMM_X86_FIRST,
	LYNX_YMM1,
	LYNX_YMM2,
	LYNX_YMM3,
	LYNX_YMM4,
	LYNX_YMM5,
	LYNX_YMM6,
	LYNX_YMM7,
	LYNX_YMM_X86_LAST=LYNX_YMM7,
	LYNX_X86_GR32_FIRST,
	LYNX_EIP=LYNX_X86_GR32_FIRST,
	LYNX_EAX,
	LYNX_EBX,
	LYNX_ECX,
	LYNX_EDX,
	LYNX_EBP,
	LYNX_ESI,
	LYNX_EDI,
	LYNX_ESP,
	LYNX_X86_GR32_LAST=LYNX_ESP,
	LYNX_X64_GR32_FIRST,
	LYNX_R8D=LYNX_X64_GR32_FIRST,
	LYNX_R9D,
	LYNX_R10D,
	LYNX_R11D,
	LYNX_R12D,
	LYNX_R13D,
	LYNX_R14D,
	LYNX_R15D,
	LYNX_X64_GR32_LAST=LYNX_R15D,
	LYNX_X86_GR16_FIRST,
	LYNX_IP=LYNX_X86_GR16_FIRST,
	LYNX_AX,
	LYNX_BX,
	LYNX_CX,
	LYNX_DX,
	LYNX_BP,
	LYNX_SI,
	LYNX_DI,
	LYNX_SP,
	LYNX_X86_GR16_LAST=LYNX_SP,
	LYNX_X64_GR16_FIRST,
	LYNX_R8W=LYNX_X64_GR16_FIRST,
	LYNX_R9W,
	LYNX_R10W,
	LYNX_R11W,
	LYNX_R12W,
	LYNX_R13W,
	LYNX_R14W,
	LYNX_R15W,
	LYNX_X64_GR16_LAST=LYNX_R15W,
	LYNX_X86_GR8H_FIRST,
	LYNX_AH=LYNX_X86_GR8H_FIRST,
	LYNX_BH,
	LYNX_CH,
	LYNX_DH,
	LYNX_X86_GR8H_LAST=LYNX_DH,
	LYNX_X86_GR8_FIRST,
	LYNX_AL=LYNX_X86_GR8_FIRST,
	LYNX_BL,
	LYNX_CL,
	LYNX_DL,
	LYNX_X86_GR8_LAST=LYNX_DL,
	LYNX_X64_GR8_FIRST,
	LYNX_BPL=LYNX_X64_GR8_FIRST,
	LYNX_SIL,
	LYNX_DIL,
	LYNX_SPL,
	LYNX_R8B,
	LYNX_R9B,
	LYNX_R10B,
	LYNX_R11B,
	LYNX_R12B,
	LYNX_R13B,
	LYNX_R14B,
	LYNX_R15B,
	LYNX_X64_GR8_LAST=LYNX_R15B,
	LYNX_YMM_X64_FIRST,
	LYNX_YMM8=LYNX_YMM_X64_FIRST,
	LYNX_YMM9,
	LYNX_YMM10,
	LYNX_YMM11,
	LYNX_YMM12,
	LYNX_YMM13,
	LYNX_YMM14,
	LYNX_YMM15,
	LYNX_YMM_X64_LAST=LYNX_YMM15,
	LYNX_YMM_FIRST,
	LYNX_YMM16=LYNX_YMM_FIRST,
	LYNX_YMM17,
	LYNX_YMM18,
	LYNX_YMM19,
	LYNX_YMM20,
	LYNX_YMM21,
	LYNX_YMM22,
	LYNX_YMM23,
	LYNX_YMM24,
	LYNX_YMM25,
	LYNX_YMM26,
	LYNX_YMM27,
	LYNX_YMM28,
	LYNX_YMM29,
	LYNX_YMM30,
	LYNX_YMM31,
	LYNX_YMM_LAST=LYNX_YMM31,
	LYNX_XMM_X86_FIRST,
	LYNX_XMM0=LYNX_XMM_X86_FIRST,
	LYNX_XMM1,
	LYNX_XMM2,
	LYNX_XMM3,
	LYNX_XMM4,
	LYNX_XMM5,
	LYNX_XMM6,
	LYNX_XMM7,
	LYNX_XMM_X86_LAST=LYNX_XMM7,
	LYNX_XMM_X64_FIRST,
	LYNX_XMM8=LYNX_XMM_X64_FIRST,
	LYNX_XMM9,
	LYNX_XMM10,
	LYNX_XMM11,
	LYNX_XMM12,
	LYNX_XMM13,
	LYNX_XMM14,
	LYNX_XMM15,
	LYNX_XMM_X64_LAST=LYNX_XMM15,
	LYNX_XMM_FIRST,
	LYNX_XMM16=LYNX_XMM_FIRST,
	LYNX_XMM17,
	LYNX_XMM18,
	LYNX_XMM19,
	LYNX_XMM20,
	LYNX_XMM21,
	LYNX_XMM22,
	LYNX_XMM23,
	LYNX_XMM24,
	LYNX_XMM25,
	LYNX_XMM26,
	LYNX_XMM27,
	LYNX_XMM28,
	LYNX_XMM29,
	LYNX_XMM30,
	LYNX_XMM31,
	LYNX_XMM_LAST=LYNX_XMM31,
	LYNX_MMX_FIRST,
	LYNX_MM0=LYNX_MMX_FIRST,
	LYNX_MM1,
	LYNX_MM2,
	LYNX_MM3,
	LYNX_MM4,
	LYNX_MM5,
	LYNX_MM6,
	LYNX_MM7,
	LYNX_MMX_LAST=LYNX_MM7,
	LYNX_FPU_STACK_FIRST,
	LYNX_FPU_STACK=LYNX_FPU_STACK_FIRST,
	LYNX_FPU_STACK_LAST=LYNX_FPU_STACK,
	LYNX_LAST_REG=LYNX_FPU_STACK,
	LYNX_INVALID,
	LYNX_LAST=LYNX_INVALID
} LynxReg;

//Gets a string representation of the given LynxReg
const char *LynxReg2Str(LynxReg lReg);

//Gets the size of the lReg in a 64 bit architecture
uint8_t LynxRegSize(LynxReg lReg);

//Gets the size of the lReg in a 32 bit architecture
uint8_t LynxRegSize32(LynxReg lReg);

//Gets the number of bytes this register is offset from the least significant bit of the fullReg
uint8_t LynxRegOffset(LynxReg lReg);

//Gets the full register for the input register if all registers are considered
LynxReg LynxReg2FullLynxReg(LynxReg lReg);

//Gets the full register for the input register if 32bit registers are considered
LynxReg LynxReg2FullLynxIA32EReg(LynxReg lReg);

//Gets the full register for the input register if 64bit registers are considered (Excluding MIC, which is the ZMM registers)
LynxReg LynxReg2FullLynxIA32Reg(LynxReg lReg);
#endif
