/**
 * This is the driver for the trace2ascii tool. Most of the work is done by the reader. In here, we just get
 * instructions one by one and print out the information to STDOUT. It should be noted that we are currently
 * only printing out an instruction's: address, source name, function name, thread id, bytes and mnemonic.
 **/

#include <Reader.h>
#include "stdio.h"

int main(int argc, char *argv[]) {
	if(argc != 2) {
		printf("Usage: %s <trace>\n", argv[0]);
		return 1;
	}

    initReader(argv[1], 0);

	ReaderEvent curEvent;

    InsInfo info;
    initInsInfo(&info);

    while(nextEventWithCheck(&curEvent, &info)) {
        if(curEvent.type == EXCEPTION_EVENT) {
            printf("EXCEPTION %d\n", curEvent.exception.code);
        }
        else if(curEvent.type == INS_EVENT) {
            fetchInsInfo(&curEvent.ins, &info);

            printf("%d %llx %s %s ", curEvent.ins.tid, (unsigned long long) curEvent.ins.addr, fetchSrcName(&curEvent.ins), fetchFnName(&curEvent.ins));

            int i;
            for(i = 0; i < curEvent.ins.binSize; i++) {
                printf("%02x ", curEvent.ins.binary[i]);
            }
            printf("%s\n", info.mnemonic);
        }
        else {
            printf("UNKNOWN EVENT TYPE\n");
        }
    }

	closeReader();

	return 0;
}
